﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Steema.TeeChart.WPF;
using Steema.TeeChart.WPF.Drawing;
using Steema.TeeChart.WPF.Styles;
using Steema.TeeChart.WPF.Tools;

namespace TChartTest
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        readonly Color _color1 = Color.FromRgb(65, 177, 225);
        readonly Color _color2 = Color.FromRgb(156, 214, 15);
        readonly Color _color3 = Color.FromRgb(142, 196, 65);

        readonly Color _grey1 = Color.FromRgb(192, 192, 192);
        readonly Color _grey2 = Color.FromRgb(224, 224, 224);

        private readonly Color _alarmColor = Colors.Red;
        private readonly Color _alarmHighlightColor = Colors.DarkOrange;

        private const string MovementActivityLabel = "Movment";
        private const string OffPanelActivityLabel = "Off Panel";
        private const string InBedActivityLabel = "In Bed";

        private FastLine _hrSeries;
        private FastLine _rrSeries;
        private Gantt _activitySeries;
        private double[] Yvalues1, Yvalues2, GYvalues;
        private DateTime[] Xvalues1, Xvalues2, GXvalues; 
        public MainWindow()
        {
            InitializeComponent();
            InitializeChart();
            AddSampleData();
        }


        private void InitializeChart()
        {
            const int hrHeightPercent = 36;
            const int rrHeightPercent = 36;
            const int hrRrMarginPercent = 5;
            const int rrActivityMarginPercent = 5;
         
            var labelFont = new ChartFont(_tchart.Chart, new Typeface("Segoe UI"));
            _tchart.Header.Visible = false;
            _tchart.Panel.Bevel.Outer = BevelStyles.None;
            _tchart.Panel.Brush = new ChartBrush(_tchart.Chart, Colors.White);
            _tchart.Walls.Visible = false;
            _tchart.Aspect.View3D = false;
            _tchart.Legend.Visible = false;
            _tchart.Panel.MarginLeft = 65;
            _tchart.Panel.MarginUnits = PanelMarginUnits.Pixels;
            _tchart.Panning.Allow = ScrollModes.None;
            _tchart.Zoom.Direction = ZoomDirections.Horizontal;
            _tchart.Zoom.Allow = false;

            _tchart.Axes.Bottom.Grid.Visible = false;
            _tchart.Axes.Bottom.Labels.Font = labelFont;
            _tchart.Axes.Bottom.AxisPen.Color = _grey1;
            _tchart.Axes.Bottom.AxisPen.Width = 1;
            _tchart.Axes.Bottom.Automatic = false;

            var scrollPager = new ScrollPager();
            _tchart.Tools.Add(scrollPager);

            var hrAxis = new Axis
            {
                Minimum = 50,
                Maximum = 130,
                StartPosition = 0,
                EndPosition = hrHeightPercent,
                Labels = { Font = labelFont, OnAxis = true },
                Title = { Angle = 90, Caption = "HEART", Font = labelFont },
                AxisPen = { Color = _grey1, Width = 1 },
                Grid = { Color = _grey2, Style = DashStyles.Solid },
                MinorTicks = { Visible = false },
            };
            var rrAxis = new Axis
            {
                Minimum = 5,
                Maximum = 40,
                StartPosition = hrHeightPercent + hrRrMarginPercent,
                EndPosition = hrHeightPercent + rrHeightPercent + hrRrMarginPercent,
                Labels = { Font = labelFont, OnAxis = true },
                Title = { Angle = 90, Caption = "RESPIRATION", Font = labelFont },
                AxisPen = { Color = _grey1, Width = 1 },
                Grid = { Color = _grey2, Style = DashStyles.Solid },
                MinorTicks = { Visible = false }
            };
            var activityAxis = new Axis
            {
                Minimum = 0,
                Maximum = 2,
                AutomaticMaximum = false,
                AutomaticMinimum = false,
                StartPosition = hrHeightPercent + rrHeightPercent + hrRrMarginPercent + rrActivityMarginPercent,
                EndPosition = 100,
                Labels = { Font = labelFont },
                AxisPen = { Color = _grey1, Width = 1 },
                Grid = { Color = _grey2, Style = DashStyles.Solid },
                MinorTicks = { Visible = false }
            };

            _tchart.Axes.Custom.Add(hrAxis);
            _tchart.Axes.Custom.Add(rrAxis);
            _tchart.Axes.Custom.Add(activityAxis);

            // for some reason, an exception is thrown if this is set before the activity axis is added to the chart
            activityAxis.Labels.Items.Add(0, "Absent").Font = labelFont;
            activityAxis.Labels.Items.Add(1, "Off Panel").Font = labelFont;
            activityAxis.Labels.Items.Add(2, "Movement").Font = labelFont;

            //Heart
            _hrSeries = new FastLine(_tchart.Chart)
            {
                CustomVertAxis = hrAxis,
                LinePen = { Width = 2, Color = _color1 },
                TreatNulls = TreatNullsStyle.DoNotPaint,
               // DrawAllPoints=false,
            };
             _hrSeries.XValues.DateTime= true; 

            //Respiration
            _rrSeries = new FastLine(_tchart.Chart)
            {
                CustomVertAxis = rrAxis,
                LinePen = { Width = 3, Color = _color2 },
                TreatNulls = TreatNullsStyle.DoNotPaint,
                //DrawAllPoints = false,
            };
            _rrSeries.XValues.DateTime = true; 
            //((FastLine)_rrSeries).DrawAllPointsStyle = DrawAllPointsStyle.MinMax;

            _activitySeries = new Gantt(_tchart.Chart)
            {
                CustomVertAxis = activityAxis,
                ColorEach = false,
            };
            _activitySeries.Pointer.Pen.Visible = false;
            //moveSeries.Pointer.Visible = false;


            //_currentSelectedAlarmEventCursor = new CursorTool(_tchart.Chart)
            //{
            //    Style = CursorToolStyles.Vertical,
            //    Pen = { Style = DashStyles.Solid, Color = Colors.Gold, Width = 3 },
            //    Active = false
            //};
        }

        void AddSampleData()
        {
            int secBetweenTicks = 8;
            //int nValues = 300;
            int nValues = 3600 / secBetweenTicks;
            int iOffPanelStart = 190;
            int iAbsentStart = 200;
            int iAbsentEnd = 240;
            DateTime startTime = new DateTime(2012, 10, 12, 8, 5, 0);
            DateTime endTime = startTime.AddSeconds(nValues * secBetweenTicks);
            Random r = new Random(0);

            Debug.WriteLine("Total sample data values: " + nValues);

            _tchart.Axes.Bottom.SetMinMax(startTime, endTime);
            _tchart.AutoRepaint = false; 
            _hrSeries.Add(startTime, 90);
            for (int i = 1; i < nValues; i++)
            {
                if (i >= iOffPanelStart && i < iAbsentEnd)
                    _hrSeries.Add(_hrSeries.XValues.AsDateTime(_hrSeries.Count - 1).AddSeconds(secBetweenTicks), null);
                else if (i == iAbsentEnd)
                    _hrSeries.Add(_hrSeries.XValues.AsDateTime(_hrSeries.Count - 1).AddSeconds(secBetweenTicks), 90);
                else if (i < iOffPanelStart || i > iAbsentEnd)
                    _hrSeries.Add(_hrSeries.XValues.AsDateTime(_hrSeries.Count - 1).AddSeconds(secBetweenTicks), _hrSeries.YValues[_hrSeries.Count - 1] + r.Next(3) - 1);
              
            }
            _rrSeries.Add(startTime, 12);
            for (int i = 1; i < nValues; i++)
            {
                if (i >= iOffPanelStart && i < iAbsentEnd)
                    _rrSeries.Add(_rrSeries.XValues.AsDateTime(_rrSeries.Count - 1).AddSeconds(secBetweenTicks), null);
                else if (i == iAbsentEnd)
                    _rrSeries.Add(_rrSeries.XValues.AsDateTime(_rrSeries.Count - 1).AddSeconds(secBetweenTicks), 12);
                else if (i < iOffPanelStart || i > iAbsentEnd)
                    _rrSeries.Add(_rrSeries.XValues.AsDateTime(_rrSeries.Count - 1).AddSeconds(secBetweenTicks), _rrSeries.YValues[_rrSeries.Count - 1] + r.Next(3) - 1);
            }

            ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            // Add many bars to the Gantt series
            //
            // this kills performance.  i need to support large data sets, and even with down sampling I really would like to get decent performance with thousands of bars.
            //
            for (int i = 0; i < 12000; i++)
            {
                _activitySeries.Add(startTime, startTime.AddSeconds(i * 10), 2, Colors.Red);

            }
            ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            _activitySeries.Add(startTime.AddSeconds(iOffPanelStart * secBetweenTicks), startTime.AddSeconds(iAbsentStart * secBetweenTicks), 1, _color2);
            _activitySeries.Add(startTime.AddSeconds(iAbsentStart * secBetweenTicks), startTime.AddSeconds(iAbsentEnd * secBetweenTicks), 0, _color3);

            _activitySeries.ConnectingPen.Visible = false;
            _tchart.AutoRepaint = true; 
            CursorTool cursor1 = new CursorTool(_tchart.Chart)
            {
                Style = CursorToolStyles.Vertical,
                Pen = { Style = DashStyles.Solid, Color = _alarmColor, Width = 3 }
            };
            cursor1.XValue = 1;
            cursor1.YValue = 0;
            cursor1.XValue = Utils.DateTime(startTime.AddSeconds(iAbsentStart * secBetweenTicks));
        }


    }
}
